#include <iostream>
#include <errlog.h>
#include <astrotime.h>

#include "deftypes.h"
#include "omicron.h"

const std::string start_phase = "Power on: ";
const std::string sect_comp = "Components";
const unsigned int short_delay = 3;
const double partially_open = 0.5;

AstroDateTime ut;

Omicron::Omicron( Config& cfg, bool m ) try : man_mode(m) {
//    delay_to_druid = cfg.inp<unsigned int>( sect_comp, "Dome", "StartDelay");
//    delay_from_druid = cfg.inp<unsigned int>( sect_comp, "Dome", "HaltDelay");
    fraction_open = cfg.inp<double>( sect_comp, "Dome", "OpenFraction");
    ClientSocket::AddrType domeIP = IP4Addr( cfg.inp<std::string>( sect_comp,  "Dome", "TheHost"),
                                             cfg.inp<unsigned int>( sect_comp, "Dome", "ThePort") );
    dome = std::auto_ptr<ClientHandler>( new ClientHandler( domeIP ) );
//    ClientSocket::AddrType irIP   = IP4Addr( cfg.inp<std::string>( sect_comp,  "IRSky", "TheHost"),
//                                             cfg.inp<unsigned int>( sect_comp, "IRSky", "ThePort") );
//    irsens = std::auto_ptr<ClientHandler>( new ClientHandler( irIP ) );
    host_checked = cfg.inp<std::string>( sect_comp, "Druid", "TheHost");
    start();
}
catch(const std::exception& e ) {
    ErrlogSingl::instance().write( std::string("ASM start: ") + e.what() );
    throw ErrFat( ERR_DEV_FUNC, "ASM connection fails" );
}

Omicron::~Omicron() {
    finish();
}

bool Omicron::testDruid() {
    std::string req = "ping -c1 " + host_checked + " > /dev/null";
    return system( req.c_str() ) == 0;
}

bool Omicron::haltDruid() try {
            // While empty
    return false;
}
catch(const std::exception& e ) {
    ErrlogSingl::instance().write( std::string("ASM finish: ") + e.what() );
    return false;
}

bool Omicron::start() try {
    if( !testDruid() ) throw ErrFat( ERR_DEV_FUNC, "Druid connection fails" );
    ErrlogSingl::instance().write( "Druid is running" );
//    irsens->command( "init" );
//    if( irsens->status( "READY" ) ) ErrlogSingl::instance().write( "IR sensor starts" );
                                            //        !!!
    while( !dome->get_status( "READY" ) ) {
        if( dome->status( "ERFAT" ) ) throw ErrFat( ERR_DEV_FUNC, "Dome component fails" );
        sleep( 4*short_delay );
    }
    ErrlogSingl::instance().write( "Dome component is ready" );

    point<bool> ready = position_check( point<double>( fraction_open, fraction_open ) );
    while( !ready.x() || !ready.y() ) {
        if( !ready.x() ) move_east( fraction_open );
        if( !ready.y() ) move_west( fraction_open );
        ready = position_check( point<double>( fraction_open, fraction_open ) );   
    }
    ErrlogSingl::instance().write( "Dome is partially opened" );
/*    
    dome->command( "get dome" );
    if( dome->get_value<std::string>( "dome" ) != "OPENED" ) {
        dome->command( "get position" );
        double e = dome->get_value<double>( "SIDE_A" );
        double w = dome->get_value<double>( "SIDE_B" );
        if( e  < fraction_open ) dome->command( "run sideA=0.55" );
        if( w  < fraction_open ) dome->command( "run sideB=0.55" );            
        dome->command( "get position" );
        if( dome->get_value<double>( "SIDE_A" ) > fraction_open && dome->get_value<double>( "SIDE_B" ) > fraction_open ) {
            ErrlogSingl::instance().write( "Dome is partially opened" );
        } else throw ErrFat( ERR_DEV_FUNC, "Dome opening fails" );
    } else ErrlogSingl::instance().write( "Dome is opened" );
 */
    return true;
}
catch(const std::exception& e ) {
    ErrlogSingl::instance().write( std::string("ASM start: ") + e.what() );
    dome->command( "run dome=close" );
    throw ErrFat( ERR_DEV_FUNC, "ASM start fails" );
    return false;
}

point<bool> Omicron::position_check( const point<double>& wanted ) {
        dome->command( "get position" );
        point<double> pos = point<double>( dome->get_value<double>( "SIDE_A" ), dome->get_value<double>( "SIDE_B" ) );
        return point<bool>( std::abs(wanted.x()-pos.x()) < 0.03 , std::abs(wanted.y()-pos.y()) < 0.03 );
}

void Omicron::move_west( double fract) try {
        std::ostringstream ostr;
        ostr << "run sideB=" << std::fixed << std::setprecision(2) << fract;
        dome->command( ostr.str() );
}
catch(const std::exception& e ) {
    ErrlogSingl::instance().write( std::string("Dome: ") + e.what() );
}

void Omicron::move_east( double fract) try {
        std::ostringstream ostr;
        ostr << "run sideA=" << std::fixed << std::setprecision(2) << fract;
        dome->command( ostr.str() );
}
catch(const std::exception& e ) {
    ErrlogSingl::instance().write( std::string("Dome: ") + e.what() );
}

double Omicron::set_west( double fract) try {
    dome->command( "get position" );
    double pos = dome->get_value<double>( "SIDE_B" );
    double initial_pos = pos;
    while( std::abs( pos - fract ) > 0.03) {
        move_west( fract );
        dome->command( "get position" );
        pos = dome->get_value<double>( "SIDE_B" );
    }
    ErrlogSingl::instance().write( "West side is succefully tuned" );
    return initial_pos;        
}
catch(const std::exception& e ) {
    ErrlogSingl::instance().write( std::string("Dome: ") + e.what() );
    return -1.0;
}

double Omicron::set_east( double fract ) try {
    dome->command( "get position" );
    double pos = dome->get_value<double>( "SIDE_A" );
    double initial_pos = pos;
    while( std::abs( pos - fract ) > 0.03) {
        move_east( fract );
        dome->command( "get position" );
        pos = dome->get_value<double>( "SIDE_A" );
    }
    ErrlogSingl::instance().write( "East side is succefully tuned" );
    return initial_pos;        
}
catch(const std::exception& e ) {
    ErrlogSingl::instance().write( std::string("Dome: ") + e.what() );
    return -1.0;
}


bool Omicron::finish() try {
    ErrlogSingl::instance().write( "Finish of the observation run" );
    dome->command( "run dome=close" );
    dome->command( "get dome" );
    if( dome->get_value<std::string>( "dome" ) != "CLOSED" ) {
        sleep( short_delay );
        dome->command( "get position" );        
        dome->command( "run dome=close" );
        dome->command( "get dome" );
    }  
//    irsens->command( "park" );
//    if( irsens->status( "PARKED" ) ) ErrlogSingl::instance().write( "IR sensor stopped" );
    return true;
}
catch(const std::exception& e ) {
    ErrlogSingl::instance().write( std::string("ASM finish: ") + e.what() );
    return false;
}




